package fr.atlantique.imt.inf211.jobmngt.controller;

import java.util.List;
import java.util.Optional;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import fr.atlantique.imt.inf211.jobmngt.dao.AppUserDao;
import fr.atlantique.imt.inf211.jobmngt.dao.CandidateDao;
import fr.atlantique.imt.inf211.jobmngt.entity.Candidate;

@RestController
@RequestMapping("/candidates")
public class CandidateController {

    @Autowired
    private CandidateDao candidateDao;

    @Autowired
    private AppUserDao appUserDao;

    /**
     * Ajouter un candidat
     * @param candidate Le candidat envoyé en JSON
     * @return Message de confirmation ou d'erreur
     */
    @Transactional
    @PostMapping("/create")
    public String addCandidate(@RequestBody Candidate candidate) {
        try {
            if (candidate.getAppUser() == null || candidate.getAppUser().getMail() == null) {
                return "Erreur : les informations utilisateur sont invalides.";
            }
            appUserDao.persist(candidate.getAppUser());
            candidate.setId(candidate.getAppUser().getId());
            candidateDao.persist(candidate);
            System.out.println("✅ Candidat ajouté : " + candidate.getFirstName() + " " + candidate.getLastName());
            return "Candidat ajouté avec succès.";
        } catch (Exception e) {
            System.err.println("❌ Erreur lors de l'ajout du candidat : " + e.getMessage());
            return "Erreur lors de l'ajout du candidat.";
        }
    }

    /**
     * Récupérer tous les candidats
     * @return Liste des candidats
     */
    @Transactional(readOnly = true)
    @GetMapping("/list")
    public List<Candidate> listCandidates() {
        List<Candidate> candidates = candidateDao.findAll("id", "asc");
        System.out.println("📌 Nombre de candidats trouvés : " + candidates.size());
        return candidates;
    }

    /**
     * Trouver un candidat par ID
     * @param id L'ID du candidat
     * @return Le candidat correspondant
     */
    @Transactional(readOnly = true)
    @GetMapping("/find/{id}")
    public Optional<Candidate> getCandidateById(@PathVariable int id) {
        Candidate candidate = candidateDao.findById(id);
        if (candidate != null) {
            System.out.println("📌 Candidat trouvé : " + candidate.getFirstName() + " " + candidate.getLastName());
            return Optional.of(candidate);
        } else {
            System.out.println("❌ Aucun candidat trouvé avec l'ID : " + id);
            return Optional.empty();
        }
    }

    /**
     * Supprimer un candidat par ID
     * @param id L'ID du candidat
     * @return Message de confirmation ou d'erreur
     */
    @Transactional
    @DeleteMapping("/delete/{id}")
    public String deleteCandidate(@PathVariable int id) {
        Candidate candidate = candidateDao.findById(id);
        if (candidate != null) {
            candidateDao.remove(candidate);
            System.out.println("🗑️ Candidat supprimé : " + id);
            return "Candidat supprimé avec succès.";
        } else {
            return "❌ Erreur : Candidat non trouvé.";
        }
    }
}
