package fr.atlantique.imt.inf211.jobmngt.controller;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.servlet.ModelAndView;

import fr.atlantique.imt.inf211.jobmngt.dao.CandidateDao;
import fr.atlantique.imt.inf211.jobmngt.dao.CompanyDao;

@Controller
public class PagesController {

    @Autowired
    private CompanyDao companyDao;

    @Autowired
    private CandidateDao candidateDao;

    /**
     * Page d'accueil avec affichage des statistiques.
     * @return ModelAndView avec le nombre d'entreprises et de candidats
     */
    @GetMapping("/")
    public ModelAndView welcomePage() {
        ModelAndView modelAndView = new ModelAndView("index");

        try {
            long countCompanies = companyDao.count();
            long countCandidates = candidateDao.count();

            modelAndView.addObject("countCompanies", countCompanies);
            modelAndView.addObject("countCandidates", countCandidates);

            System.out.println("📊 Statistiques : " + countCompanies + " entreprises, " + countCandidates + " candidats.");
        } catch (Exception e) {
            System.err.println("❌ Erreur lors du chargement des statistiques : " + e.getMessage());
            modelAndView.addObject("countCompanies", "Erreur");
            modelAndView.addObject("countCandidates", "Erreur");
        }

        return modelAndView;
    }
}
