package fr.atlantique.imt.inf211.jobmngt.dao;

import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import fr.atlantique.imt.inf211.jobmngt.entity.Application;
import jakarta.persistence.EntityManager;
import jakarta.persistence.PersistenceContext;

@Repository
public class ApplicationDao {

    private static final Logger logger = Logger.getLogger(ApplicationDao.class.getName());

    @PersistenceContext
    private EntityManager entityManager;

    @Transactional
    public void persist(Application application) {
        logger.log(Level.INFO, "Persisting Application: " + application);
        entityManager.persist(application);
    }

    @Transactional
    public void remove(Application application) {
        logger.log(Level.INFO, "Removing Application: " + application);
        if (!entityManager.contains(application)) {
            application = entityManager.merge(application);
        }
        entityManager.remove(application);
    }

    @Transactional
    public Application merge(Application application) {
        logger.log(Level.INFO, "Merging Application: " + application);
        return entityManager.merge(application);
    }

    @Transactional(readOnly = true)
    public Application findById(int id) {
        return entityManager.find(Application.class, id);
    }

    @Transactional(readOnly = true)
    public Long count() {
        return entityManager.createQuery("SELECT COUNT(a) FROM Application a", Long.class)
                            .getSingleResult();
    }

    @Transactional(readOnly = true)
    public List<Application> findAll(String sort, String order) {
        String allowedSortFields = "id,cv,appdate";
        if (!allowedSortFields.contains(sort)) {
            logger.log(Level.WARNING, "Invalid sort field: " + sort);
            sort = "id";
        }
        String jpql = "SELECT a FROM Application a ORDER BY a." + sort + ("asc".equalsIgnoreCase(order) ? " ASC" : " DESC");
        return entityManager.createQuery(jpql, Application.class).getResultList();
    }
}
