package fr.atlantique.imt.inf211.jobmngt.dao;

import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import fr.atlantique.imt.inf211.jobmngt.entity.ApplicationMessage;
import jakarta.persistence.EntityManager;
import jakarta.persistence.PersistenceContext;

@Repository
public class ApplicationMessageDao {

    private static final Logger logger = Logger.getLogger(ApplicationMessageDao.class.getName());

    @PersistenceContext
    private EntityManager entityManager;

    @Transactional
    public void persist(ApplicationMessage applicationMessage) {
        logger.log(Level.INFO, "Persisting ApplicationMessage: " + applicationMessage);
        entityManager.persist(applicationMessage);
    }

    @Transactional
    public void remove(ApplicationMessage applicationMessage) {
        logger.log(Level.INFO, "Removing ApplicationMessage: " + applicationMessage);
        if (!entityManager.contains(applicationMessage)) {
            applicationMessage = entityManager.merge(applicationMessage);
        }
        entityManager.remove(applicationMessage);
    }

    @Transactional(readOnly = true)
    public ApplicationMessage findById(int id) {
        return entityManager.find(ApplicationMessage.class, id);
    }

    @Transactional(readOnly = true)
    public Long count() {
        return entityManager.createQuery("SELECT COUNT(am) FROM ApplicationMessage am", Long.class)
                            .getSingleResult();
    }

    @Transactional(readOnly = true)
    public List<ApplicationMessage> findAll(String sort, String order) {
        // Vérification des champs valides pour éviter les erreurs SQL
        String allowedSortFields = "id,sentdate,message";
        if (!allowedSortFields.contains(sort)) {
            logger.log(Level.WARNING, "Invalid sort field: " + sort);
            sort = "id"; // Défaut
        }
        String jpql = "SELECT am FROM ApplicationMessage am ORDER BY am." + sort + ("asc".equalsIgnoreCase(order) ? " ASC" : " DESC");
        return entityManager.createQuery(jpql, ApplicationMessage.class).getResultList();
    }
}
