package fr.atlantique.imt.inf211.jobmngt.dao;

import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import fr.atlantique.imt.inf211.jobmngt.entity.Company;
import jakarta.persistence.EntityManager;
import jakarta.persistence.PersistenceContext;

@Repository
public class CompanyDao {

    private static final Logger logger = Logger.getLogger(CompanyDao.class.getName());

    @PersistenceContext
    private EntityManager entityManager;

    @Transactional
    public void persist(Company company) {
        logger.log(Level.INFO, "Persisting Company: " + company.getName());
        entityManager.persist(company);
    }

    @Transactional
    public void remove(Company company) {
        logger.log(Level.INFO, "Removing Company with id: " + company.getId());
        if (!entityManager.contains(company)) {
            company = entityManager.merge(company);
        }
        entityManager.remove(company);
    }

    @Transactional
    public Company merge(Company company) {
        logger.log(Level.INFO, "Merging Company with id: " + company.getId());
        return entityManager.merge(company);
    }

    @Transactional(readOnly = true)
    public Company findById(Integer id) {
        if (id == null) {
            logger.log(Level.WARNING, "findById() called with null id");
            return null;
        }
        logger.log(Level.INFO, "Finding Company with id: " + id);
        return entityManager.find(Company.class, id);
    }

    @Transactional(readOnly = true)
    public Long count() {
        return entityManager.createQuery("SELECT COUNT(c) FROM Company c", Long.class)
                            .getSingleResult();
    }

    @Transactional(readOnly = true)
    public List<Company> findAll(String sort, String order) {
        // Vérification des champs valides
        String allowedSortFields = "id,name,city";
        if (!allowedSortFields.contains(sort)) {
            logger.log(Level.WARNING, "Invalid sort field: " + sort);
            sort = "id"; // Valeur par défaut
        }

        String jpql = "SELECT c FROM Company c ORDER BY c." + sort + ("asc".equalsIgnoreCase(order) ? " ASC" : " DESC");
        return entityManager.createQuery(jpql, Company.class).getResultList();
    }
}
