package fr.atlantique.imt.inf211.jobmngt.dao;

import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import fr.atlantique.imt.inf211.jobmngt.entity.OfferMessage;
import jakarta.persistence.EntityManager;
import jakarta.persistence.PersistenceContext;
import jakarta.persistence.TypedQuery;

@Repository
public class OfferMessageDao {

    private static final Logger logger = Logger.getLogger(OfferMessageDao.class.getName());

    @PersistenceContext
    private EntityManager entityManager;

    @Transactional
    public void persist(OfferMessage offerMessage) {
        logger.log(Level.INFO, "Persisting OfferMessage: " + offerMessage.getMessage());
        entityManager.persist(offerMessage);
    }

    @Transactional
    public void remove(OfferMessage offerMessage) {
        logger.log(Level.INFO, "Removing OfferMessage with id: " + offerMessage.getId());
        if (!entityManager.contains(offerMessage)) {
            offerMessage = entityManager.merge(offerMessage);
        }
        entityManager.remove(offerMessage);
    }

    @Transactional(readOnly = true)
    public OfferMessage findById(int id) {
        if (id <= 0) {
            logger.log(Level.WARNING, "findById() called with invalid id: " + id);
            return null;
        }
        logger.log(Level.INFO, "Finding OfferMessage with id: " + id);
        return entityManager.find(OfferMessage.class, id);
    }

    @Transactional(readOnly = true)
    public Long count() {
        return entityManager.createQuery("SELECT COUNT(o) FROM OfferMessage o", Long.class)
                            .getSingleResult();
    }

    @Transactional(readOnly = true)
    public List<OfferMessage> findAll(String sort, String order) {
        // Vérification des champs autorisés
        String allowedSortFields = "id,sentdate";
        if (!allowedSortFields.contains(sort)) {
            logger.log(Level.WARNING, "Invalid sort field: " + sort);
            sort = "id"; // Valeur par défaut
        }

        String orderDirection = "DESC";
        if ("asc".equalsIgnoreCase(order)) {
            orderDirection = "ASC";
        }

        String jpql = "SELECT om FROM OfferMessage om ORDER BY om." + sort + " " + orderDirection;
        TypedQuery<OfferMessage> query = entityManager.createQuery(jpql, OfferMessage.class);

        logger.log(Level.INFO, "Executing findAll() query: " + jpql);
        return query.getResultList();
    }
}
