package fr.atlantique.imt.inf211.jobmngt.dao;

import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import fr.atlantique.imt.inf211.jobmngt.entity.QualificationLevel;
import jakarta.persistence.EntityManager;
import jakarta.persistence.PersistenceContext;
import jakarta.persistence.TypedQuery;

@Repository
public class QualificationLevelDao {

    private static final Logger logger = Logger.getLogger(QualificationLevelDao.class.getName());

    @PersistenceContext
    private EntityManager entityManager;

    @Transactional(readOnly = true)
    public Long count() {
        return entityManager.createQuery("SELECT COUNT(q) FROM QualificationLevel q", Long.class)
                            .getSingleResult();
    }

    @Transactional(readOnly = true)
    public List<QualificationLevel> findAll(String sort, String order) {
        // Vérification des champs autorisés pour éviter toute injection SQL
        String allowedSortFields = "id,label";
        if (!allowedSortFields.contains(sort)) {
            logger.log(Level.WARNING, "Invalid sort field: " + sort);
            sort = "id"; // Valeur par défaut
        }

        String orderDirection = "DESC";
        if ("asc".equalsIgnoreCase(order)) {
            orderDirection = "ASC";
        }

        String jpql = "SELECT q FROM QualificationLevel q ORDER BY q." + sort + " " + orderDirection;
        TypedQuery<QualificationLevel> query = entityManager.createQuery(jpql, QualificationLevel.class);

        logger.log(Level.INFO, "Executing findAll() query: " + jpql);
        return query.getResultList();
    }

    @Transactional
    public void persist(QualificationLevel qualificationLevel) {
        logger.log(Level.INFO, "Persisting QualificationLevel: " + qualificationLevel.getLabel());
        entityManager.persist(qualificationLevel);
    }

    @Transactional
    public void remove(QualificationLevel qualificationLevel) {
        logger.log(Level.INFO, "Removing QualificationLevel with id: " + qualificationLevel.getId());
        if (!entityManager.contains(qualificationLevel)) {
            qualificationLevel = entityManager.merge(qualificationLevel);
        }
        entityManager.remove(qualificationLevel);
    }

    @Transactional
    public QualificationLevel merge(QualificationLevel qualificationLevel) {
        logger.log(Level.INFO, "Merging QualificationLevel: " + qualificationLevel.getLabel());
        return entityManager.merge(qualificationLevel);
    }

    @Transactional(readOnly = true)
    public QualificationLevel findById(int id) {
        if (id <= 0) {
            logger.log(Level.WARNING, "findById() called with invalid id: " + id);
            return null;
        }
        logger.log(Level.INFO, "Finding QualificationLevel with id: " + id);
        return entityManager.find(QualificationLevel.class, id);
    }
}
