package fr.atlantique.imt.inf211.jobmngt.dao;

import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import fr.atlantique.imt.inf211.jobmngt.entity.Sector;
import jakarta.persistence.EntityManager;
import jakarta.persistence.PersistenceContext;
import jakarta.persistence.TypedQuery;

@Repository
public class SectorDao {

    private static final Logger logger = Logger.getLogger(SectorDao.class.getName());

    @PersistenceContext
    private EntityManager entityManager;

    @Transactional(readOnly = true)
    public Long count() {
        return entityManager.createQuery("SELECT COUNT(s) FROM Sector s", Long.class).getSingleResult();
    }

    @Transactional(readOnly = true)
    public List<Sector> findAll(String sort, String order) {
        // Vérification des champs autorisés pour éviter toute injection SQL
        String allowedSortFields = "id,label";
        if (!allowedSortFields.contains(sort)) {
            logger.log(Level.WARNING, "Invalid sort field: " + sort);
            sort = "id"; // Valeur par défaut
        }

        String orderDirection = "DESC";
        if ("asc".equalsIgnoreCase(order)) {
            orderDirection = "ASC";
        }

        String jpql = "SELECT s FROM Sector s ORDER BY s." + sort + " " + orderDirection;
        TypedQuery<Sector> query = entityManager.createQuery(jpql, Sector.class);

        logger.log(Level.INFO, "Executing findAll() query: " + jpql);
        return query.getResultList();
    }

    @Transactional
    public void persist(Sector sector) {
        logger.log(Level.INFO, "Persisting Sector: " + sector.getLabel());
        entityManager.persist(sector);
    }

    @Transactional
    public void remove(Sector sector) {
        logger.log(Level.INFO, "Removing Sector with id: " + sector.getId());
        if (!entityManager.contains(sector)) {
            sector = entityManager.merge(sector);
        }
        entityManager.remove(sector);
    }

    @Transactional
    public Sector merge(Sector sector) {
        logger.log(Level.INFO, "Merging Sector: " + sector.getLabel());
        return entityManager.merge(sector);
    }

    @Transactional(readOnly = true)
    public Sector findById(int id) {
        if (id <= 0) {
            logger.log(Level.WARNING, "findById() called with invalid id: " + id);
            return null;
        }
        logger.log(Level.INFO, "Finding Sector with id: " + id);
        return entityManager.find(Sector.class, id);
    }
}
